//main.cpp

#include "d3dUtility.h"
#include <vector>
#include "ObjectX.h"
#include "Pick.h"
//
// Globals
//

IDirect3DDevice9* Device = 0; 

const int Width  = 640;
const int Height = 480;

ObjectX *xAirplane;
ObjectX *xDwarf;
ObjectX *xTiger;

Pick *pick;

float y = 0.0f;
float x = 0.0f;

//
// Framework functions
//
bool Setup()
{
	xAirplane = new ObjectX(Device);
	xDwarf = new ObjectX(Device);
	xTiger = new ObjectX(Device);

	xAirplane->setFile("airplane 2.x");
	xDwarf->setFile("Dwarf.x");
	xTiger->setFile("tiger.x");

	pick = new Pick(Device);

	//
	// Set texture filters.
	//

	Device->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	Device->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	Device->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_POINT);

	// 
	// Set Lights.
	//

	D3DXVECTOR3 dir(1.0f, -1.0f, 1.0f);
	D3DXCOLOR col(1.0f, 1.0f, 1.0f, 1.0f);
	D3DLIGHT9 light = d3d::InitDirectionalLight(&dir, &col);

	Device->SetLight(0, &light);
	Device->LightEnable(0, true);
	Device->SetRenderState(D3DRS_NORMALIZENORMALS, true);
	Device->SetRenderState(D3DRS_SPECULARENABLE, true);

	//
	// Set camera.
	//

	D3DXVECTOR3 pos(4.0f, 4.0f, -13.0f);
	D3DXVECTOR3 target(0.0f, 0.0f, 0.0f);
	D3DXVECTOR3 up(0.0f, 1.0f, 0.0f);

	D3DXMATRIX V;
	D3DXMatrixLookAtLH(
		&V,
		&pos,
		&target,
		&up);

	Device->SetTransform(D3DTS_VIEW, &V);

	//
	// Set projection matrix.
	//

	D3DXMATRIX proj;
	D3DXMatrixPerspectiveFovLH(
			&proj,
			D3DX_PI * 0.5f, // 90 - degree
			(float)Width / (float)Height,
			1.0f,
			1000.0f);
	Device->SetTransform(D3DTS_PROJECTION, &proj);

	return true;
}

void Cleanup()
{
	xAirplane->xfile->Cleanup();
	xDwarf->xfile->Cleanup();
	xTiger->xfile->Cleanup();
}

bool Display(float timeDelta)
{
	if( Device )
	{
		//
		// Update: Rotate the mesh.
		//
		xAirplane->setTran(-7.0f, 0.0f, x);
		xDwarf->setTran(7.0f, 0.0f, x);
		xTiger->setTran(0.0f, 0.0f, x);

		xAirplane->setYRot(y);
		xDwarf->setYRot(y);
		xTiger->setYRot(y);

		//
		// Render
		//

		Device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0);
		Device->BeginScene();

		xAirplane->display();
		xDwarf->display();
		xTiger->display();

		Device->EndScene();
		Device->Present(0, 0, 0, 0);
	}
	return true;
}

//
// WndProc
//
LRESULT CALLBACK d3d::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_DESTROY:
		::PostQuitMessage(0);
		break;
		
	case WM_KEYDOWN:
		if( wParam == VK_ESCAPE )
			::DestroyWindow(hwnd);
		switch(wParam){
		case VK_UP:
			x += 0.1f;
			break;
		case VK_RIGHT:
			y -= 0.2f;
			break;
		case VK_LEFT:
			y += 0.2f;
			
			break;
		case VK_DOWN:
			x -= 0.1f;
			
			break;
		}
		break;

	case WM_LBUTTONDOWN:
		xAirplane->setBSphere();
		xDwarf->setBSphere();
		xTiger->setBSphere();
		
		if( pick->pickTest(LOWORD(lParam), HIWORD(lParam), &xAirplane->BSphere) )
			::MessageBox(0, "Airplane Hit!", "HIT", 0);

		else if( pick->pickTest(LOWORD(lParam), HIWORD(lParam), &xDwarf->BSphere) )
			::MessageBox(0, "Dwarf Hit!", "HIT", 0);
		
		else if( pick->pickTest(LOWORD(lParam), HIWORD(lParam), &xTiger->BSphere) )
			::MessageBox(0, "Tiger Hit!", "HIT", 0);

		break;
	}
	
	return ::DefWindowProc(hwnd, msg, wParam, lParam);
}

//
// WinMain
//
int WINAPI WinMain(HINSTANCE hinstance,
				   HINSTANCE prevInstance, 
				   PSTR cmdLine,
				   int showCmd)
{
	if(!d3d::InitD3D(hinstance,
		Width, Height, true, D3DDEVTYPE_HAL, &Device))
	{
		::MessageBox(0, "InitD3D() - FAILED", 0, 0);
		return 0;
	}
		
	if(!Setup())
	{
		::MessageBox(0, "Setup() - FAILED", 0, 0);
		return 0;
	}

	d3d::EnterMsgLoop( Display );

	Cleanup();

	Device->Release();

	return 0;
}


